@@ -15,6 +15,7 @@
 # include <sys/types.h>
 # include <sys/socket.h>
 # include <netinet/in.h>
+# include <netinet/tcp.h>
 # include <arpa/inet.h>
 # include <unistd.h>
 # include <fcntl.h>
@@ -30,8 +31,8 @@ bool
 net_init(void) {
 #ifdef _WIN32
     WSADATA wsa;
-    int res = WSAStartup(MAKEWORD(2, 2), &wsa) < 0;
-    if (res < 0) {
+    int res = WSAStartup(MAKEWORD(1, 1), &wsa);
+    if (res) {
         LOGE("WSAStartup failed with error %d", res);
         return false;
     }
@@ -273,6 +274,22 @@ net_close(sc_socket socket) {
 #endif
 }
 
+bool
+net_set_tcp_nodelay(sc_socket socket, bool tcp_nodelay) {
+    sc_raw_socket raw_sock = unwrap(socket);
+
+    int value = tcp_nodelay ? 1 : 0;
+    int ret = setsockopt(raw_sock, IPPROTO_TCP, TCP_NODELAY,
+                         (const void *) &value, sizeof(value));
+    if (ret == -1) {
+        net_perror("setsockopt(TCP_NODELAY)");
+        return false;
+    }
+
+    assert(ret == 0);
+    return true;
+}
+
 bool
 net_parse_ipv4(const char *s, uint32_t *ipv4) {
     struct in_addr addr;
